"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _mobx = require("mobx");

var _cache = require("./utils/cache");

var _utils = require("./utils");

var _windowFocus = _interopRequireDefault(require("./utils/windowFocus"));

var _windowVisible = _interopRequireDefault(require("./utils/windowVisible"));

var _debounce = _interopRequireDefault(require("./utils/debounce"));

var _throttle = _interopRequireDefault(require("./utils/throttle"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

var noop = function noop() {};
/**
 * mobx 异步函数的状态管理
 */


var AsyncState = /*#__PURE__*/function () {
  /**
   * 创建一个mobx异步函数管理，自动观察 loading data error 数据。
   * 
   * @param {Promise<any>} asyncFn - 异步函数
   * @param {object} options - 配置项
   */
  function AsyncState(asyncFn) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, AsyncState);

    // 异步函数
    this.asyncFn = asyncFn; // 配置项

    this.options = _objectSpread({
      defaultParams: [],
      defaultLoading: false,
      // initialData,
      onSuccess: noop,
      onError: noop,
      autoRun: true,
      // formatResult,
      cacheKey: '',
      cacheTime: 5 * 60 * 1000,
      pollingInterval: 0,
      pollingWhenHidden: true,
      refreshOnWindowFocus: false,
      focusTimespan: 5000
    }, options);
    var _this$options = this.options,
        defaultParams = _this$options.defaultParams,
        defaultLoading = _this$options.defaultLoading,
        initialData = _this$options.initialData,
        cacheKey = _this$options.cacheKey,
        debounceInterval = _this$options.debounceInterval,
        throttleInterval = _this$options.throttleInterval,
        focusTimespan = _this$options.focusTimespan,
        pollingInterval = _this$options.pollingInterval,
        refreshOnWindowFocus = _this$options.refreshOnWindowFocus; // 计数，用于防止多次请求倒置问题

    this.__requestCount = 0; // 参数

    this.params = Array.isArray(defaultParams) ? defaultParams : defaultParams ? [defaultParams] : []; // 返回值

    this.originData = cacheKey ? (0, _cache.getCache)(cacheKey) : initialData; // 错误

    this.originError = null; // 标识正在执行

    this.loading = defaultLoading; // 防抖运行

    this.debounceRun = debounceInterval ? (0, _debounce["default"])(this._run.bind(this), debounceInterval) : undefined; // 节流运行

    this.throttleRun = throttleInterval ? (0, _throttle["default"])(this._run.bind(this), throttleInterval) : undefined; // 轮询定时器

    this.pollingTimer = null; // 视窗获取焦点标识触发轮询标识

    this.pollingWhenVisibleFlag = false; // 延迟loading定时器

    this.loadingDelayTimer = null; // 取消订阅集合

    this.unsubscribe = []; // 订阅页面显示时轮询

    if (pollingInterval) {
      this.unsubscribe.push((0, _windowVisible["default"])(this.rePolling.bind(this)));
    } // 订阅屏幕聚焦时请求


    if (refreshOnWindowFocus) {
      var throttleRefresh = (0, _throttle["default"])(this.refresh.bind(this), focusTimespan);
      this.unsubscribe.push((0, _windowFocus["default"])(throttleRefresh));
    }

    this._run = this._run.bind(this);
    this.run = this.run.bind(this);
    this.cancel = this.cancel.bind(this);
    this.rePolling = this.rePolling.bind(this);
    this.mutate = this.mutate.bind(this);
    this.refresh = this.refresh.bind(this);
    this.destroy = this.destroy.bind(this); // 自动执行

    if (this.options.autoRun) {
      this.run.apply(this, _toConsumableArray(this.params));
    }
  } // 转换data


  _createClass(AsyncState, [{
    key: "data",
    get: function get() {
      return (0, _mobx.toJS)(this.originData);
    } // 转换error

  }, {
    key: "error",
    get: function get() {
      return (0, _mobx.toJS)(this.originError);
    } // 重启轮询

  }, {
    key: "rePolling",
    value: function rePolling() {
      if (this.pollingWhenVisibleFlag) {
        this.pollingWhenVisibleFlag = false;
        this.refresh();
      }
    } // 触发异步函数

  }, {
    key: "run",
    value: function run() {
      if (this.debounceRun) {
        this.debounceRun.apply(this, arguments);
        return Promise.resolve(null);
      }

      if (this.throttleRun) {
        this.throttleRun.apply(this, arguments);
        return Promise.resolve(null);
      }

      this.__requestCount += 1;
      return this._run.apply(this, arguments);
    }
  }, {
    key: "_run",
    value: function _run() {
      var _this = this;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var _this$options2 = this.options,
          loadingDelay = _this$options2.loadingDelay,
          formatResult = _this$options2.formatResult,
          cacheKey = _this$options2.cacheKey,
          cacheTime = _this$options2.cacheTime,
          onSuccess = _this$options2.onSuccess,
          onError = _this$options2.onError,
          pollingInterval = _this$options2.pollingInterval,
          pollingWhenHidden = _this$options2.pollingWhenHidden; // 取消轮询定时器

      if (this.pollingTimer) {
        clearTimeout(this.pollingTimer);
      } // 取消延迟loading


      if (this.loadingDelayTimer) {
        clearTimeout(this.loadingDelayTimer);
      }

      var currentCount = this.__requestCount;
      this.params = args;
      this.loading = !loadingDelay; // 设置延迟loading定时器

      if (loadingDelay) {
        this.loadingDelayTimer = setTimeout(function () {
          _this.loading = true;
        }, loadingDelay);
      } else {
        this.loadingDelayTimer = null;
      }

      return new Promise(function (resolve, reject) {
        _this.asyncFn.apply(_this, args).then(function (res) {
          if (currentCount === _this.__requestCount) {
            if (_this.loadingDelayTimer) {
              clearTimeout(_this.loadingDelayTimer);
            }

            var fmtData = typeof formatResult === 'function' ? formatResult(res) : res;
            (0, _mobx.runInAction)(function () {
              _this.loading = false;
              _this.originData = fmtData;
              _this.originError = null;
            });

            if (cacheKey) {
              (0, _cache.setCache)(cacheKey, fmtData, cacheTime);
            }

            typeof onSuccess === 'function' && onSuccess(fmtData, _this.params);
            resolve(fmtData);
          }
        })["catch"](function (err) {
          if (currentCount === _this.__requestCount) {
            if (_this.loadingDelayTimer) {
              clearTimeout(_this.loadingDelayTimer);
            }

            (0, _mobx.runInAction)(function () {
              _this.loading = false;
              _this.originError = err;
            });
            typeof onError === 'function' && onError(err, _this.params);
            reject(err);
          }
        })["finally"](function () {
          if (currentCount === _this.__requestCount) {
            // 轮询
            if (pollingInterval) {
              // 视窗失去焦点
              if (!(0, _utils.isDocumentVisible)() && !pollingWhenHidden) {
                _this.pollingWhenVisibleFlag = true;
                return;
              }

              _this.pollingTimer = setTimeout(function () {
                _this.run.apply(_this, args);
              }, pollingInterval);
            }
          }
        });
      });
    } // 取消执行异步函数

  }, {
    key: "cancel",
    value: function cancel() {
      if (this.debounceRun) {
        this.debounceRun.cancel();
      }

      if (this.throttleRun) {
        this.throttleRun.cancel();
      } // 取消轮询定时器


      if (this.pollingTimer) {
        clearTimeout(this.pollingTimer);
      } // 取消延迟loading


      if (this.loadingDelayTimer) {
        clearTimeout(this.loadingDelayTimer);
      }

      this.pollingWhenVisibleFlag = false;
      this.__requestCount += 1;
      this.loading = false;
    } // 销毁

  }, {
    key: "destroy",
    value: function destroy() {
      this.cancel(); // 取消订阅

      this.unsubscribe.forEach(function (s) {
        return s();
      });
    } // 用之前的参数，重新执行异步函数

  }, {
    key: "refresh",
    value: function refresh() {
      return this.run.apply(this, _toConsumableArray(this.params));
    } // 直接修改data

  }, {
    key: "mutate",
    value: function mutate(newData) {
      if (typeof newData === 'function') {
        this.originData = newData(this.data);
      } else {
        this.originData = newData;
      }
    }
  }]);

  return AsyncState;
}();

var _default = (0, _mobx.decorate)(AsyncState, {
  loading: _mobx.observable,
  originError: _mobx.observable,
  originData: _mobx.observable,
  error: _mobx.computed,
  data: _mobx.computed,
  _run: _mobx.action,
  cancel: _mobx.action,
  refresh: _mobx.action,
  mutate: _mobx.action
});

exports["default"] = _default;