"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = throttle;

// 节流
// 多次调用在一定时间间隔内仅执行一次
function throttle(fn, timespan) {
  // 最后一次执行时间
  var last; // 定时器

  var timer = null; // 最后执行时的参数

  var lastArgs; // 正在等待执行

  var pendding = false; // 取消执行

  function cancel() {
    clearTimeout(timer);
    pendding = false;
  }

  function throttled() {
    lastArgs = arguments;

    if (pendding) {
      return;
    }

    var now = +new Date();
    var context = this;

    if (last && now < last + timespan) {
      pendding = true;
      timer = setTimeout(function () {
        last = now;
        pendding = false;
        fn.apply(context, lastArgs);
      }, timespan);
    } else {
      last = now;
      fn.apply(context, lastArgs);
    }
  }

  throttled.cancel = cancel;
  return throttled;
}